function createFinder(keyword) {
  var finder = {
    count: 0,
    textMatch: {
      caseSensitive: false,
      index: -1,
      remaining: '',
      init: function(keyword) {
        if (this.caseSensitive) {
          keyword = keyword.toLowerCase();
        }
        keyword = keyword.replace(/&/, '&amp;').replace(/</, '&lt;').replace(/>/, '&gt;');
        this.actual = keyword.split('');
      },
      getMatched: function() {
        var openTags = [];
        var closeTags = [];
        var index = this.remaining.indexOf('<');
        while (index !== -1) {
          var tag = "";
          var open = true;
          for (var i = index + 1; i < this.remaining.length; i++) {
            var c = this.remaining.chartAt(i);
            if (i === index + 1) {
              open = c !== '/';
            }
            if (c == ' ' || c == '>') {
              if (open) {
                openTags[openTags.length] = tag;
              } else {
                tag = tag.substring(1);
                if (openTags.length === 0) {
                  closeTags[closeTags.length] = tag;
                } else {
                  var t = openTags[openTags.length - 1];
                  if (t === tag) {
                    openTags.pop();
                  } else {
                    closeTags[closeTags.length] = tag;
                  }
                }
              }
              index = this.remaining.indexOf('/', i + 1);
              break;
            } else {
              tag += c;
            }
          }
        }
        var result = "";
        closeTags.forEach(function(each) {
          result += '</' + each + '>';
        });
        result += '<span class="find">';
        closeTags.forEach(function(each) {
          result += '<' + each + '>';
        });
        result += this.remaining;
        openTags.forEach(function(each) {
          result += '</' + each + '>';
        });
        result += '</span>';
        openTags.forEach(function(each) {
          result += '<' + each + '>';
        });
        return result;
      },
      getRemaining: function() {
        return this.remaining;
      },
      reset: function() {
        this.index = -1;
        this.remaining = '';
      },
      add: function(c, ignore) {
        var compare = this.caseSensitive ? c : c.toLowerCase();
        if (this.index === -1) {
          if (!ignore && compare === this.actual[0]) {
            this.remaining += c;
            this.index = 0;
            return this.actual.length == 1 ? 'MATCHED' : 'IGNORE';
          } else {
            return 'ADD';
          }
        } else {
          this.remaining += c;
          if (ignore) {
            return 'IGNORE';
          } else if (compare === this.actual[++this.index]) {
            return (this.index === this.actual.length - 1) ? 'MATCHED' : 'IGNORE';
          } else {
            return 'UNMATCHED';
          }
        }
      }
    },
    find: function(content) {
      if (!content) {
        return '';
      }
      var result = '';
      content = content.replace(/(&nbsp;){1}/, ' ');
      var ignore = false;
      for (var i = 0; i < content.length; i++) {
        var each = content.charAt(i);
        if (!ignore && each === '<') {
          ignore = true;
        }
        switch (this.textMatch.add(each, ignore)) {
        case 'ADD':
          result += each;
          break;
        case 'MATCHED':
          result += this.textMatch.getMatched();
          this.textMatch.reset();
          this.count++;
          break;
        case 'UNMATCHED':
          result += this.textMatch.getRemaining();
          this.textMatch.reset();
          break;
        }
        if (ignore && each == '>') {
          ignore = false;
        }
      }
      result += this.textMatch.getRemaining();
      this.textMatch.reset();
      return result;
    },
    getCount: function() {
      return this.count;
    },
    reset: function() {
      this.count = 0;
      this.textMatch.reset();
    },
    search: function(content) {
      if (!this.keyword) {
        return {
          html: '',
          count: 0
        };
      }
      var count = 0;
      var results = [];
      $.each(content.elements, function(p, each) {
        var text = each.text;
        var sb = this.find(text);
        var c = this.getCount();
        if (c > 0) {
          count += c;
          if (p === 0) { // 장 제목과 부제목
            sb = '<div id="' + each.id + '" class="element chapter">' + sb + '</div>';
          }
          results.push({
            id: each.id,
            text: sb
          });
        }
        this.reset();
      }.bind(this));
      html = '';
      if (count > 0) {
        html = '<h1 class="chapter" data-id="' + content.id + '"';
        if (content.partTitle) {
          html += ' data-part-title="' + content.partTitle + '"';
        }
        html += '>' + content.title + '<span class="count">' + count + '</span></h1>';
        html += '<div id="content" class="r_content">';
        $.each(results, function(i, each) {
          html += each.text;
        });
        html += '</div>';
      }
      return {
        html: html,
        count: count
      };
    }
  };

  keyword = keyword.toLowerCase();
  finder.textMatch.init(keyword);
  finder.keyword = keyword;
  return finder;
};
